@@ -1,23 +1,20 @@
 import os
-import six
 import uuid
 import shutil
-import random
 import logging
-import traceback
 from copy import deepcopy
 from tempfile import mkdtemp
-from datetime import datetime, timedelta
 from contextlib import contextmanager
-import time
+from servicelayer.cache import make_key
+from servicelayer.util import load_json, dump_json
 
-from memorious.core import manager, storage, celery, session
-from memorious.core import datastore, local_queue
-from memorious.model import Result, Tag, Event
+from memorious.core import manager, storage, conn
+from memorious.core import datastore
+from memorious.model import Event, Queue, Crawl
 from memorious.logic.http import ContextHttp
-from memorious.logic.rate_limit import rate_limiter, RateLimitException
-from memorious.util import make_key, random_filename
-from memorious import settings, signals
+from memorious.logic.check import ContextCheck
+from memorious.util import random_filename
+from memorious.exc import QueueTooBigError
 
 
 class Context(object):
@@ -34,26 +31,29 @@ class Context(object):
         self.log = logging.getLogger('%s.%s' % (crawler.name, stage.name))
         self.http = ContextHttp(self)
         self.datastore = datastore
+        self.check = ContextCheck(self)
 
     def get(self, name, default=None):
         """Get a configuration value and expand environment variables."""
         value = self.params.get(name, default)
-        if isinstance(value, six.string_types):
+        if isinstance(value, str):
             value = os.path.expandvars(value)
         return value
 
-    def emit(self, rule='pass', stage=None, data={}, delay=None):
+    def emit(self, rule='pass', stage=None, data={}, delay=None,
+             optional=False):
         """Invoke the next stage, either based on a handling rule, or by calling
         the `pass` rule by default."""
         if stage is None:
             stage = self.stage.handlers.get(rule)
+        if optional and stage is None:
+            return
         if stage is None or stage not in self.crawler.stages:
             self.log.info("No next stage: %s (%s)" % (stage, rule))
             return
         state = self.dump_state()
         delay = delay or self.crawler.delay
-        Result.save(self.crawler, self.stage.name, stage, data)
-        handle.apply_async((state, stage, data), countdown=delay)
+        Queue.queue(stage, state, data)
 
     def recurse(self, data={}, delay=None):
         """Have a stage invoke itself with a modified set of arguments."""
@@ -64,54 +64,57 @@ class Context(object):
     def execute(self, data):
         """Execute the crawler and create a database record of having done
         so."""
+        if Crawl.is_aborted(self.crawler, self.run_id):
+            return
 
         try:
-            signals.operation_start.send(self)
-            self.log.debug('Running: %s', self.stage.name)
+            Crawl.operation_start(self.crawler, self.stage, self.run_id)
+            self.log.info('[%s->%s(%s)]: %s',
+                          self.crawler.name,
+                          self.stage.name,
+                          self.stage.method_name,
+                          self.run_id)
             return self.stage.method(self, data)
+        except QueueTooBigError as qtbe:
+            self.emit_warning(str(qtbe))
         except Exception as exc:
-            # this should clear results and tags created by this op
-            # TODO: should we also use transactions on the datastore?
-            session.rollback()
             self.emit_exception(exc)
         finally:
-            signals.operation_end.send(self)
+            Crawl.operation_end(self.crawler, self.run_id)
             shutil.rmtree(self.work_path)
-            # Save the results and events created in this op
-            session.commit()
 
-    def emit_warning(self, message, type=None, details=None, *args):
+    def emit_warning(self, message, type=None, *args):
         if len(args):
             message = message % args
         self.log.warning(message)
-        return Event.save(self.crawler.name,
-                          self.stage.name,
+        return Event.save(self.crawler,
+                          self.stage,
                           Event.LEVEL_WARNING,
                           self.run_id,
-                          error_type=type,
-                          error_message=message,
-                          error_details=details)
+                          error=type,
+                          message=message)
 
     def emit_exception(self, exc):
         self.log.exception(exc)
-        return Event.save(self.crawler.name,
-                          self.stage.name,
+        return Event.save(self.crawler,
+                          self.stage,
                           Event.LEVEL_ERROR,
                           self.run_id,
-                          error_type=exc.__class__.__name__,
-                          error_message=six.text_type(exc),
-                          error_details=traceback.format_exc())
+                          error=exc.__class__.__name__,
+                          message=str(exc))
 
     def set_tag(self, key, value):
-        return Tag.save(self.crawler, key, value)
+        data = dump_json(value)
+        key = make_key(self.crawler, "tag", key)
+        return conn.set(key, data, ex=self.crawler.expire)
 
     def get_tag(self, key):
-        tag = Tag.find(self.crawler, key)
-        if tag is not None:
-            return tag.value
+        value = conn.get(make_key(self.crawler, "tag", key))
+        if value is not None:
+            return load_json(value)
 
     def check_tag(self, key):
-        return Tag.exists(self.crawler, key)
+        return conn.exists(make_key(self.crawler, "tag", key))
 
     def skip_incremental(self, *criteria):
         """Perform an incremental check on a set of criteria.
@@ -130,14 +133,7 @@ class Context(object):
         if key is None:
             return False
 
-        # this is used to re-run parts of a scrape after a certain interval,
-        # e.g. half a year, or a year
-        since = None
-        if self.crawler.expire > 0:
-            delta = timedelta(days=self.crawler.expire)
-            since = datetime.utcnow() - delta
-
-        if Tag.exists(self.crawler, key, since=since):
+        if self.check_tag(key):
             return True
 
         self.set_tag(key, None)
@@ -154,7 +150,7 @@ class Context(object):
         path = random_filename(self.work_path)
         try:
             with open(path, 'wb') as fh:
-                if isinstance(data, six.text_type):
+                if isinstance(data, str):
                     data = data.encode(encoding)
                 if data is not None:
                     fh.write(data)
@@ -166,7 +162,7 @@ class Context(object):
                 pass
 
     @contextmanager
-    def load_file(self, content_hash, file_name=None):
+    def load_file(self, content_hash, file_name=None, read_mode='rb'):
         file_path = storage.load_file(content_hash,
                                       file_name=file_name,
                                       temp_path=self.work_path)
@@ -174,7 +170,7 @@ class Context(object):
             yield None
         else:
             try:
-                with open(file_path, 'r') as fh:
+                with open(file_path, mode=read_mode) as fh:
                     yield fh
             finally:
                 storage.cleanup_file(content_hash,
@@ -199,33 +195,3 @@ class Context(object):
 
     def __repr__(self):
         return '<Context(%r, %r)>' % (self.crawler, self.stage)
-
-
-@celery.task(bind=True, max_retries=None)
-def handle(task, state, stage, data):
-    """Execute the operation, rate limiting allowing."""
-    context = Context.from_state(state, stage)
-    if context.crawler.disabled:
-        return
-
-    if context.stage.rate_limit:
-        try:
-            with rate_limiter(context):
-                if settings.EAGER:
-                    local_queue.queue_operation(context, data)
-                else:
-                    context.execute(data)
-                return
-        except RateLimitException:
-            delay = max(1, 1.0/context.stage.rate_limit)
-            delay = random.randint(1, int(delay))
-            context.log.info("Rate limit exceeded, delaying %d sec.", delay)
-            time.sleep(delay)
-
-    if settings.EAGER:
-        # If celery is running in eager mode, put the crawler in a
-        # Queue. Then we get to execute them sequentially and avoid
-        # recursion errors.
-        local_queue.queue_operation(context, data)
-    else:
-        context.execute(data)
