@@ -3,4 +3,4 @@ from pprint import pformat
 
 def inspect(context, data):
     context.log.info(pformat(data))
-    context.emit(data=data)
\ No newline at end of file
+    context.emit(data=data, optional=True)
