@@ -3,10 +3,11 @@ import json
 
 import pytest
 
-from memorious.operations import (
-    fetch, session, parse, seed, sequence, dates, enumerate, directory
-)
-from memorious.core import session as db_session
+from memorious.operations.fetch import fetch, session
+from memorious.operations.parse import parse
+from memorious.operations.initializers import seed, sequence, dates, enumerate
+from memorious.operations.store import directory
+from memorious.core import connect_redis
 
 
 @pytest.mark.parametrize("url,call_count", [
@@ -56,14 +57,15 @@ def test_parse(context, mocker):
             "url": "http://www.iana.org/domains/example"
         })
 
-    # Clear existing tags
-    db_session.rollback()
+    # cleanup tags
+    conn = connect_redis()
+    conn.flushall()
 
     with mocker.patch.object(context, "emit"):
         context.http.result = None
         context.params["store"] = None
         parse(context, data)
-        assert context.emit.call_count == 2
+        assert context.emit.call_count == 2, data
 
 
 def test_seed(context, mocker):
@@ -148,6 +150,6 @@ def test_directory(context):
     assert os.path.exists(raw_file_path)
 
     with open(meta_file_path, "rb") as fh:
-        assert json.load(fh) == data
+        assert json.load(fh)['content_hash'] == data['content_hash']
     with open(raw_file_path, "rb") as fh:
-        assert fh.read() == b'{\n  "user-agent": "Memorious Test"\n}\n'
+        assert b'"user-agent": "Memorious Test"' in fh.read()
